@@ -1,6 +1,6 @@
 /* match.s -- optional optimized asm version of longest match in deflate.c
 
-   Copyright (C) 2002, 2006, 2009-2013 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2006, 2009-2022 Free Software Foundation, Inc.
    Copyright (C) 1992-1993 Jean-loup Gailly
 
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /*
- * The 68020 version has been written by Francesco Potorti` <pot@cnuce.cnr.it>
+ * The 68020 version has been written by Francesco Potortì <pot@cnuce.cnr.it>
  * with adaptations by Carsten Steger <stegerc@informatik.tu-muenchen.de>,
  * Andreas Schwab <schwab@lamothe.informatik.uni-dortmund.de> and
  * Kristoffer Eriksson <ske@pkmab.se>
@@ -49,7 +49,11 @@
   error: DYN_ALLOC not yet supported in match.s
 #endif
 
-#if defined(i386) || defined(_I386) || defined(__i386) || defined(__i386__)
+/* On x86-64, Sun C 5.13 (Oracle Solaris Studio 12.4) 'cc -E -m64'
+   defines i386 when compiling .s or .S files!  Luckily it also
+   defines __x86_64__.  See Bug#23133.  */
+#if ((defined i386 || defined _I386 || defined __i386 || defined __i386__) \
+     && !defined __x86_64__)
 
 /* This version is for 386 Unix or OS/2 in 32 bit mode.
  * Warning: it uses the AT&T syntax: mov source,dest
@@ -227,7 +231,7 @@ maxmatch:
 #  define predec(An)		-(An)
 #  define postinc(An)		(An)+
 
-#else /* default style (Sun 3, NeXT, Amiga, Atari) */
+#else /* default style (Sun 3, NeXT, Atari) */
 
 #  define GLOBAL(symbol)	.globl	symbol
 #  define TEXT			.text
@@ -399,8 +403,8 @@ L__return:
 /*
  * 'longest_match.S' (assembly program for gzip for the IA-64 architecture)
  *
- * Optimised for McKinley, but with Merced-compatibility, such as MIB+MIB, used wherever
- * possible.
+ * Optimized for McKinley, but with Merced-compatibility, such as
+ * MIB+MIB, used wherever possible.
  *
  * Copyright: Sverre Jarp (HP Labs) 2001-2002
  *
@@ -720,7 +724,7 @@ longest_match:
 .only3:
 {.mib	// Cycle 0 (short)
         cmp.gt.unc p0,p_nbs=s_vlen,s_vbestlen		// (len > best_len) ?
-(p_nbs)	br.cond.dpnt.many .next_iter			// if not, re-iternate
+(p_nbs)	br.cond.dpnt.many .next_iter			// if not, reiterate
         ;;
 }{.mmi	// Cycle 1 (short)
         ld4 s_tm7=[s_anicematch] 			// nice_match
